import db from '../db.js';
import fs from 'fs/promises';
import config from '../config.js';
import { createHash } from 'crypto';
import captcha from '../controllers/captcha.js';

function base64ToArrayBuffer(base64) {
    var binaryString = atob(base64);
    var bytes = new Uint8Array(binaryString.length);
    for (var i = 0; i < binaryString.length; i++) {
        bytes[i] = binaryString.charCodeAt(i);
    }
    return bytes.buffer;
}

class CaptchaService {
    async new(image, solution, submitter) {
        const b64 = image.replace(/^data:image\/jpeg;base64,/, "");
        const arrayBuffer = base64ToArrayBuffer(b64);
        const buffer = Buffer.from(arrayBuffer);
        const hash = createHash('md5').update(buffer).digest('hex');
        await fs.writeFile(`${config.data_dir}/${hash}.jpeg`, b64, 'base64');
        await db.query("INSERT INTO captchas (hash, solution, submitter) VALUES ($1, $2, $3)", [hash, solution, submitter]);
    }

    async get(id) {
        const captcha = (await db.query("SELECT hash, solution FROM captchas WHERE id = $1", [id])).rows[0];
        if (captcha == undefined) return undefined;

        const path = `${config.data_dir}/${captcha.hash}.jpeg`;

        const image = Buffer.from(await fs.readFile(path)).toString('base64');
        return {"image": image, "solution": captcha.solution, "hash": captcha.hash};
    }

    async get_all() {
        return (await db.query("SELECT id, hash FROM captchas")).rows
    }
}

export default new CaptchaService();